#!/usr/bin/python3
# -*- coding: utf-8 -*-

#/*
# * Copyright (C) 2020, KylinSoft Co., Ltd.
# *
# * This program is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <https://www.gnu.org/licenses/>.
# */

import sqlite3
import os
import apt
from model.globals import Globals
# from PyQt5.QtCore import QProcess,QDir,QFileInfo,QFile,QIODevice
# from PyQt5.QtDBus import *
import dbus
from dbus.exceptions import DBusException

import time
import threading
import datetime
import configparser
import psutil
import json
import subprocess
import sys
import shutil

lock = threading.Lock()

class DbService:
    cpu = ""
    arch = ""
    def __init__(self):

        Globals.logInfo.logger.info("Start initializing data")
        if(self.checkDataUpdate() == False):
            return
        self.updatecount = 0
        self.aptCache = None
        cacheFile = os.path.join(Globals.installPackagePath, "uksc.db")      #安装路径的数据库
        srcFile = os.path.join(Globals.userCacheUkscDir, "uksc.db")            #从服务端获取数据更新到该数据库
        procFile = os.path.join(Globals.userCacheUkscDir, "uksc_pre_processing.db")     #处理中的数据库
        destFile = os.path.join(Globals.userCacheUkscDir, "uksc_pre_back.db")        #若前端界面启动，最终处理结束的数据库备份
        readFile = os.path.join(Globals.userCacheUkscDir, "uksc_pre.db")        #前端界面读取使用的数据库

        #reloadCache
        self.reloadCache()

        # 通过dbus判断数据是否处于更新中
        try:
            # 连接到 D-Bus 系统总线
            bus = dbus.SessionBus()
        except DBusException as e:
            # Globals.logError.logger.error("dbus exception: %s" % str(e))
            print("DBus exception: ", e)
            # 假设 finishDataPreprocessing 是一个方法，用于处理数据预处理完成后的逻辑
            # self.finishDataPreprocessing(False, "0002")
            return None  # 或者其他适当的错误处理

        dbus_server = "com.kylin.softwarecenter.getsearchresults"
        dbus_interface = "com.kylin.getsearchresults"
        dbus_path = "/com/kylin/softwarecenter/getsearchresults"
        reply = False

        try:
            # 获取远程 D-Bus 服务的对象
            remote_object = bus.get_object(dbus_server, dbus_path)

            # 获取接口并调用方法
            iface = dbus.Interface(remote_object, dbus_interface)
            try:
                # 调用 D-Bus 方法
                result = iface.dataUpdateInProgress()
                # 假设方法返回一个布尔值表示数据是否正在更新
                reply = result  # 将结果赋值给 reply 变量
                # Globals.logInfo.logger.info("dataUpdateInProgress Method reply:%s" % str(reply))
                print("dataUpdateInProgress Method reply:", reply)
            except DBusException as error:
                # Globals.logError.logger.error("DBus method call failed: %s" % str(error))
                print("DBus method call failed: ", error)
            except Exception as e:
                # Globals.logError.logger.error("An unexpected error occurred: %s" % str(e))
                print("An unexpected error occurred: ", e)
        except DBusException as e:
            # Globals.logError.logger.error("Failed to create D-Bus interface: %s" % str(e))
            print("Failed to create D-Bus interface: ", e)
        if reply == True:
            if(os.path.exists(destFile) == False or os.path.exists(readFile) == False):
                #没有预处理完的数据库，优先处理安装路径的数据库
                if not os.path.exists(cacheFile):
                    Globals.CONNECT_DB = False
                    if (Globals.debugSwitch):
                        Globals.logError.logger.error("error with db file")
                    return
                else:
                    os.system("cp " + cacheFile + " " + procFile)
            else:
                #已有预处理完的数据库，就等数据更新结束
                while(1):
                    time.sleep(2)
                    try:
                        # 获取远程 D-Bus 服务的对象
                        remote_object = bus.get_object(dbus_server, dbus_path)

                        # 获取接口
                        iface = dbus.Interface(remote_object, dbus_interface)

                        # 调用 D-Bus 方法
                        try:
                            result = iface.dataUpdateInProgress()
                            # 假设方法返回一个布尔值表示数据是否正在更新
                            reply = result
                            # Globals.logInfo.logger.info("dataUpdateInProgress Method reply:%s" % str(reply))
                            print("dataUpdateInProgress Method reply:", reply)
                        except DBusException as error:
                            # Globals.logError.logger.error("DBus method call failed: %s" % str(error))
                            print("DBus method call failed:", error)
                            # 可以选择在这里处理特定的 D-Bus 错误，或者继续循环
                        except Exception as e:
                            # Globals.logError.logger.error("An unexpected error occurred while calling DBus method: %s" % str(e))
                            print("An unexpected error occurred while calling DBus method:", e)
                            # 出现意外错误时，可以选择退出循环或进行其他错误处理
                    except DBusException as e:
                        # Globals.logError.logger.error("Failed to create D-Bus interface: %s" % str(e))
                        print("Failed to create D-Bus interface:", e)
                        # 出现 D-Bus 连接错误时，可以选择退出循环或进行其他错误处理

                        # 如果数据更新已经结束，则退出循环
                    if reply == False:
                        break


                if not os.path.exists(procFile ):
                    self.copyDbFile(srcFile,cacheFile,procFile)
                else:
                    os.remove(procFile)
                    self.copyDbFile(srcFile, cacheFile, procFile)
        else:
            if not os.path.exists(procFile ):
                self.copyDbFile(srcFile,cacheFile,procFile)
            else:
                os.remove(procFile)
                self.copyDbFile(srcFile, cacheFile, procFile)

        try:
            self.connect = sqlite3.connect(procFile, timeout=30.0, check_same_thread=False)
            self.cursor = self.connect.cursor()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
        if(os.path.exists(Globals.userConfFilePath)):
            try:
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                self.cpu = cf.get("essential-information","cpu")
                self.arch = cf.get("essential-information","arch")
            except:
                pass
        if(self.cpu == ""):
            self.getCpu()
        if(self.arch == ""):
            self.getArch()
        self.initAptCache()
        self.queryApplicationList()
        self.queryAppTag()
        self.queryAdList()
        self.queryAdLocation()
        Globals.logInfo.logger.info("Initialize data completed")

        if self.checkProcessIsRunning():
            # 不进行数据库替换，检查并处理destFile
            if os.path.exists(destFile):
                os.remove(destFile)
            if shutil.copy(procFile, destFile):
                if os.path.exists(procFile):
                    os.remove(procFile)
        else:
            # 进行数据库替换
            if os.path.exists(readFile):
                os.remove(readFile)
            if (os.path.exists(procFile)):
                if shutil.copy(procFile, readFile):
                    if os.path.exists(procFile):
                        os.remove(procFile)
            pass
        self.changeInfo()
        #需要增加一个判断数据同步结束的信号
        self.finishDataPreprocessing(True,"")

    def checkDataUpdate(self):
        destFile = os.path.join(Globals.userCacheUkscDir, "uksc_pre_back.db")        #若前端界面启动，最终处理结束的数据库备份
        readFile = os.path.join(Globals.userCacheUkscDir, "uksc_pre.db")        #前端界面读取使用的数据库
        if(os.path.exists(destFile) == False and os.path.exists(readFile) == False):
            return True
        #--------判断源更新----------
        result = str(subprocess.getoutput("LC_ALL=en_US stat /var/lib/apt/lists | grep Change"))
        recentChangeTime = ""     #获取源最近更新时间
        recordTime = ""       #记录的时间
        #获取源最近更新时间
        if(result.replace("Change:","") != ""):
            recentChangeTime = result.replace("Change:","").replace("+0800","").strip(" ").split(".")[0]
        # 获取记录的源上次更新时间
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if(cf.has_option("essential-information","source_updateTime")):
                    recordTime = cf.get("essential-information", "source_updateTime")
                if (recentChangeTime != recordTime):
                    #如果检测到记录的源更新时间与实际更新时间不一致则进行数据预处理
                    # cf.set("essential-information", "source_updateTime",recentChangeTime)
                    # with open(Globals.userConfFilePath, "w+") as f:
                    #     cf.write(f)
                    Globals.logInfo.logger.info("apt source changed time and record time:%s" % str(recentChangeTime+ " " + recordTime))
                    return True
                else:
                    #否则再进行其他条件判断
                    pass
            except Exception as e:
                print("}{}{}{}",str(e))
                return True
        Globals.logInfo.logger.info("apt source changed time and record time:%s" % str(recentChangeTime+ " " + recordTime))
        #----------判断服务端数据更新---------
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                dataUpdate = "false"
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if(cf.has_option("table-updateTime","date_update")):
                    dataUpdate = cf.get("table-updateTime", "date_update").lower()
                if (dataUpdate == "true"):
                    #如果检测到数据更新
                    # cf.set("table-updateTime","date_update","false")
                    # with open(Globals.userConfFilePath, "w+") as f:
                    #     cf.write(f)
                    Globals.logInfo.logger.info("data updates:%s" % str(dataUpdate))
                    return True
                else:
                    #否则再进行其他条件判断
                    pass
                Globals.logInfo.logger.info("data updates:%s" % str(dataUpdate))
            except Exception as e:
                print("}{}{}{}",str(e))
                return True
        #----------判断应用安装--------
        result = str(subprocess.getoutput("LC_ALL=en_US stat /var/log/dpkg.log | grep Change"))
        # 将输出按行分割成列表
        # read_text = result.stdout.splitlines()
        recentChangeTime = ""  # 获取源最近更新时间
        recordTime = ""  # 记录的时间
        # 获取源最近更新时间
        if (result.replace("Change:", "") != ""):
            recentChangeTime = result.replace("Change:", "").replace("+0800", "").strip(" ").split(".")[0]
        # 获取记录的源上次更新时间
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if (cf.has_option("essential-information", "dpkg_updateTime")):
                    recordTime = cf.get("essential-information", "dpkg_updateTime")
                if (recentChangeTime != recordTime):
                    # 如果检测到记录的源更新时间与实际更新时间不一致则进行数据预处理
                    # cf.set("essential-information", "dpkg_updateTime", recentChangeTime)
                    # with open(Globals.userConfFilePath, "w+") as f:
                    #     cf.write(f)

                    Globals.logInfo.logger.info("dpkg log changed time:%s" % str(recentChangeTime+ " " + recordTime))
                    return True
                else:
                    # 否则再进行其他条件判断
                    pass
            except Exception as e:
                print("}{}{}{}", str(e))
                return True
        Globals.logInfo.logger.info("dpkg log changed time:%s" % str(recentChangeTime+ " " + recordTime))
        return False

    def changeInfo(self):
        #--------判断源更新----------
        result = str(subprocess.getoutput("LC_ALL=en_US stat /var/lib/apt/lists | grep Change"))
        recentChangeTime = ""     #获取源最近更新时间
        recordTime = ""       #记录的时间
        #获取源最近更新时间
        if(result.replace("Change:","") != ""):
            recentChangeTime = result.replace("Change:","").replace("+0800","").strip(" ").split(".")[0]
        # 获取记录的源上次更新时间
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if(cf.has_option("essential-information","source_updateTime")):
                    recordTime = cf.get("essential-information", "source_updateTime")
                if (recentChangeTime != recordTime):
                    #如果检测到记录的源更新时间与实际更新时间不一致则进行数据预处理
                    cf.set("essential-information", "source_updateTime",recentChangeTime)
                    with open(Globals.userConfFilePath, "w+") as f:
                        cf.write(f)
                else:
                    #否则再进行其他条件判断
                    pass
            except Exception as e:
                pass
        #----------判断服务端数据更新---------
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                dataUpdate = "false"
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if(cf.has_option("table-updateTime","date_update")):
                    dataUpdate = cf.get("table-updateTime", "date_update").lower()
                if (dataUpdate == "true"):
                    #如果检测到数据更新
                    cf.set("table-updateTime","date_update","false")
                    with open(Globals.userConfFilePath, "w+") as f:
                        cf.write(f)
                else:
                    #否则再进行其他条件判断
                    pass
            except Exception as e:
                pass
        #----------判断应用安装--------
        result = str(subprocess.getoutput("LC_ALL=en_US stat /var/log/dpkg.log | grep Change"))
        # 将输出按行分割成列表
        # read_text = result.stdout.splitlines()
        recentChangeTime = ""  # 获取源最近更新时间
        recordTime = ""  # 记录的时间
        # 获取源最近更新时间
        if (result.replace("Change:", "") != ""):
            recentChangeTime = result.replace("Change:", "").replace("+0800", "").strip(" ").split(".")[0]
        # 获取记录的源上次更新时间
        if (os.path.exists(Globals.userConfFilePath)):
            try:
                cf = configparser.ConfigParser()
                cf.read(Globals.userConfFilePath)
                if (cf.has_option("essential-information", "dpkg_updateTime")):
                    recordTime = cf.get("essential-information", "dpkg_updateTime")
                if (recentChangeTime != recordTime):
                    # 如果检测到记录的源更新时间与实际更新时间不一致则进行数据预处理
                    cf.set("essential-information", "dpkg_updateTime", recentChangeTime)
                    with open(Globals.userConfFilePath, "w+") as f:
                        cf.write(f)
                else:
                    # 否则再进行其他条件判断
                    pass
            except Exception as e:
                pass

    def reloadCache(self):
        packageDbusServer = "com.kylin.softwarecenter.packageInfo"
        packageDbusInterface = "com.kylin.softwarecenter.packageInfo"
        packageDbusPath = "/com/kylin/softwarecenter/packageInfo"
        reply = False
        try:
            # 连接到 D-Bus 系统总线
            bus = dbus.SessionBus()
        except DBusException as e:
            # Globals.logError.logger.error("dbus exception: %s" % str(e))
            print("DBus exception: ", e)
            # 假设 finishDataPreprocessing 是一个方法，用于处理数据预处理完成后的逻辑
            # self.finishDataPreprocessing(False, "0002")
            return None  # 或者其他适当的错误处理

        try:
            # 获取远程 D-Bus 服务的对象
            packageObject = bus.get_object(packageDbusServer, packageDbusPath)

            # 获取接口并调用方法
            iface = dbus.Interface(packageObject, packageDbusInterface)
            try:
                # 调用 D-Bus 方法
                result = iface.reloadAptCache()
                # 假设方法返回一个布尔值表示数据是否正在更新
                reply = result  # 将结果赋值给 reply 变量
                Globals.logInfo.logger.info("dataUpdateInProgress Method reply:%s" % str(reply))
                # print("reload Cache Method reply:", reply)
            except DBusException as error:
                Globals.logError.logger.error("DBus method call failed: %s" % str(error))
                # print("DBus method call failed: ", error)
            except Exception as e:
                Globals.logError.logger.error("An unexpected error occurred: %s" % str(e))
                # print("An unexpected error occurred: ", e)
        except DBusException as e:
            Globals.logError.logger.error("Failed to create D-Bus interface: %s" % str(e))
            # print("Failed to create D-Bus interface: ", e)

        return

    def finishDataPreprocessing(self,res,errorCode):
        Globals.logInfo.logger.info("finish data preprocessing Method reply:%s" % str(res))
        dbusServer = "com.kylin.softwarecenter.getsearchresults"
        dbusInterface = "com.kylin.getsearchresults"
        dbusPath = "/com/kylin/softwarecenter/getsearchresults"
        try:
            # 连接到 D-Bus 系统总线
            bus = dbus.SessionBus()
        except DBusException as e:
            Globals.logError.logger.error("dbus exception: %s" % str(e))
            # print("DBus exception: ", e)
            # 假设 finishDataPreprocessing 是一个方法，用于处理数据预处理完成后的逻辑
            # self.finishDataPreprocessing(False, "0002")
            return None  # 或者其他适当的错误处理

        try:
            # 获取远程 D-Bus 服务的对象
            packageObject = bus.get_object(dbusServer, dbusPath)

            # 获取接口并调用方法
            iface = dbus.Interface(packageObject, dbusInterface)
            try:
                # 调用 D-Bus 方法
                result = iface.data_preprocessing_status(res,errorCode)
                # 假设方法返回一个布尔值表示数据是否正在更新
                reply = result  # 将结果赋值给 reply 变量
                Globals.logInfo.logger.info("finishDataPreprocessing Method reply:%s" % str(reply))
                # print("reload Cache Method reply:", reply)
            except DBusException as error:
                Globals.logError.logger.error("DBus method call failed: %s" % str(error))
                # print("DBus method call failed: ", error)
            except Exception as e:
                Globals.logError.logger.error("An unexpected error occurred: %s" % str(e))
                # print("An unexpected error occurred: ", e)
        except DBusException as e:
            Globals.logError.logger.error("Failed to create D-Bus interface: %s" % str(e))
            # print("Failed to create D-Bus interface: ", e)


    def checkProcessIsRunning(self):
        #判断前端界面是否正在运行中
        for proc in psutil.process_iter(['name']):
            try:
                if proc.info['name'] == "kylin-software-center":
                    return True
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass
        return False

    def copyDbFile(self,srcFile,cacheFile,destFile):
        if not os.path.exists(srcFile):
            if not os.path.exists(cacheFile):
                Globals.CONNECT_DB = False
                if (Globals.debugSwitch):
                    Globals.logError.logger.error("error with db file")
                return
            else:
                os.system("cp " + cacheFile + " " + destFile)
        else:
            # 判断用户路径下的db数据库版本
            if (self.checkDbVersion(srcFile, cacheFile) == 1):
                # 进行数据库更新时间判断
                if (self.checkDbUpdateTime(srcFile, cacheFile) == 1):
                    os.system("cp " + srcFile + " " + destFile)
                else:
                    os.system("cp " + cacheFile + " " + destFile)
                pass
            elif (self.checkDbVersion(srcFile, cacheFile) == -1):
                # 判断数据库版本时数据库链接失败,该模块进程需要重新链接
                pass
            else:
                os.system("cp " + cacheFile + " " + destFile)

    def checkDbVersion(self,dbFile1,dbFile2):
        try:
            connect = sqlite3.connect(dbFile1, timeout=30.0, check_same_thread=False)
            cursor = connect.cursor()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
        try:
            connect1 = sqlite3.connect(dbFile2, timeout=30.0, check_same_thread=False)
            cursor1 = connect1.cursor()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
        res = None
        sql = "select value from dict where key='dbversion'"
        try:
            cursor.execute(sql)
            res = cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        db_version = ""
        if (res != None and res != []):
            db_version = res[0][0]
        res = None
        try:
            cursor1.execute(sql)
            res = cursor1.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        db_version1 = ""
        if (res != None and db_version1 != []):
            db_version1 = res[0][0]
        connect.close()
        connect1.close()
        if (db_version != db_version1):
            return -2
        else:
            return 1

    def checkDbUpdateTime(self,dbFile1,dbFile2):
        try:
            connect = sqlite3.connect(dbFile1, timeout=30.0, check_same_thread=False)
            cursor = connect.cursor()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
        try:
            connect1 = sqlite3.connect(dbFile2, timeout=30.0, check_same_thread=False)
            cursor1 = connect1.cursor()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
        res = None
        sql = "select value from dict where key = 'appinfo_updatetime'"
        try:
            cursor.execute(sql)
            res = cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        dbUpdateTime = ""
        if (res != None and res != []):
            dbUpdateTime = res[0][0]
        res = None
        try:
            cursor1.execute(sql)
            res = cursor1.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("Database link failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        dbUpdateTime1 = ""
        if (res != None and dbUpdateTime1 != []):
            dbUpdateTime1 = res[0][0]
        connect.close()
        connect1.close()
        if (dbUpdateTime != "" and dbUpdateTime1 != ""):
            format_pattern = "%Y-%m-%d %H:%M:%S"
            difference = (datetime.datetime.strptime(str(dbUpdateTime1),
                                                     format_pattern) - datetime.datetime.strptime(
                str(dbUpdateTime), format_pattern))
            if (difference.days <= 0):
                return 1
            else:
                return -1
        else:
            return -1

    def getCpu(self):
        cpu_model = ""
         # 执行 'cat /proc/cpuinfo' 命令并获取其输出
        result = subprocess.run(["cat", "/proc/cpuinfo"], capture_output=True, text=True, check=True)
        # 将输出按行分割成列表
        read_text = result.stdout.splitlines()
        # 遍历每一行，查找 CPU 型号信息
        for text in read_text:
            if "Hardware" in text and ":" in text:
                cpu_model = text.split(":")[1].strip()
            elif "model name" in text and ":" in text:
                cpu_model = text.split(":")[1].strip()
            
            # 如果找到了 CPU 型号信息，进行进一步处理
            if cpu_model:
                cpu_model = cpu_model.lower()
                break  # 假设我们只关心第一个找到的 CPU 型号，找到后即可退出循环
        # 根据 CPU 型号进行进一步分类
        if cpu_model:
            if "kirin990" in cpu_model or ("kirin" in cpu_model and "990" in cpu_model):
                cpu_model = "kirin990"
            elif "huaweikirin9006c" in cpu_model or ("kirin" in cpu_model and "9006c" in cpu_model):
                cpu_model = "kirin9006c"
            elif "intel" in cpu_model:
                cpu_model = "Intel"
            elif "amd" in cpu_model:
                cpu_model = "AMD"
            elif "hygon" in cpu_model:
                cpu_model = "Hygon"
            elif "zhaoxin" in cpu_model:
                cpu_model = "ZHAOXIN"
            elif ("ft2000a" in cpu_model and "4-core" in cpu_model) or "ft-2000" in cpu_model:
                cpu_model = "FT2000/4"
            elif "loongson" in cpu_model and "3a4000" in cpu_model:
                cpu_model = "loongson3A R4(Loongson3A4000)"
            elif "d2000" in cpu_model and "phytium" in cpu_model:
                cpu_model = "D2000"
            # 如果 CPU 型号包含多个部分（例如，由逗号分隔），则选择特定部分
            elif "," in cpu_model:
                cpu_model = cpu_model.split(",")[1].strip()
        self.cpu = cpu_model


    def getArch(self):
        # 创建子进程来运行命令
        process = subprocess.Popen(["dpkg", "--print-architecture"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        # 等待进程完成并获取输出
        stdout, stderr = process.communicate()
        # 检查是否有错误发生
        if process.returncode != 0:
            raise Exception(f"Error occurred: {stderr.decode().strip()}")
        # 获取命令的输出并去掉换行符
        self.arch = stdout.decode().strip()


    #init apt cache
    def initAptCache(self):
        try:
            self.aptCache = apt.Cache()
            # self.ca.open()
        except:
            self.aptCache = None

    #chack package
    def get_package_by_name(self,pkgname):
        if not pkgname:
            return None
        if self.aptCache is None:
            return None
        package = None
        try:
            package = self.aptCache[pkgname]
        except:
            package = None
        if package is not None:
            try:
                if package.candidate is None:
                    package = None
            except:
                package = None
        return package

    #检测定向上架应用
    def checkCpuModel(self, cpuModel, noCpuModel):
        hwmon_dir = "/sys/class/hwmon"
        readName = ""
        checkCpu = self.cpu
        archIn = True
        inCpuModel = True
        inNoCpuModel = False
        # 检查hwmon目录是否存在
        if os.path.exists(hwmon_dir):
            # 获取目录中的文件列表
            file_list = os.listdir(hwmon_dir)
            # 遍历文件列表
            for file_name in file_list:
                # 构造name文件的完整路径
                name_file_path = os.path.join(hwmon_dir, file_name, "name")

                # 检查name文件是否存在
                if os.path.exists(name_file_path):
                    # 打开并读取name文件的内容
                    with open(name_file_path, 'r') as file:
                        readline = file.read().strip()
                        # 检查文件中是否包含"huawei_qs"
                        if "huawei_qs" in readline:
                            checkCpu = "huawei_qs"
        if (cpuModel != "" and cpuModel != None):
            archIn = False
            inCpuModel = False
            for getCpuModel in cpuModel.split(";"):
                if (len(getCpuModel.split(":")) == 2 and (
                        self.arch.lower() == getCpuModel.split(":")[0].lower() or getCpuModel.split(":")[
                    0].lower() == "all")):
                    archIn = True
                    if (checkCpu.lower() in getCpuModel.split(":")[1].lower().split(",")):
                        inCpuModel = True
        # 说明定向上架只在某个架构或cpu显示，不包括本机
        if (archIn == False or inCpuModel == False):
            return False
        if (noCpuModel != "" and noCpuModel != None):
            archIn = False
            inNoCpuModel = False
            for getCpuModel in noCpuModel.split(";"):
                if (len(getCpuModel.split(":")) == 2 and (
                        self.arch.lower() == getCpuModel.split(":")[0].lower() or getCpuModel.split(":")[
                    0].lower() == "all")):
                    archIn = True
                    if (checkCpu.lower() in getCpuModel.split(":")[1].lower().split(",")):
                        inNoCpuModel = True
        if (((noCpuModel == "" or noCpuModel == None) and archIn == True and inCpuModel == True) or (
                noCpuModel != "" and noCpuModel != None and (archIn == False or inNoCpuModel == False))):
            return True
        return False

    #获取kare安装应用
    def getKarePackages(self):
        appList = {}
        package = self.get_package_by_name("kare")
        if(package == None or package.is_installed == False):
            return appList
        result = subprocess.run(["kare", "-l"], stdout=subprocess.PIPE)
        getResult = subprocess.run(["awk",'{print $1":"$2}'],stdout=subprocess.PIPE,input=result.stdout)
        # 将输出按行分割成列表
        read_text = getResult.stdout.splitlines()
        # 获取kare安装应用列表
        for _text in read_text:
            version = ""
            appName = ""
            text = _text.decode("utf-8")
            if(len(text.split(":")) == 2):
                infoMsg = {}
                version = text.split(":")[1].strip()
                appName = text.split(":")[0].strip()
                infoMsg["version"] = version
                infoMsg["packageName"] = appName
                appList[appName] = infoMsg
        return appList


    # 预处理application表格,ashStatus灰度应用状态
    def queryApplicationList(self):
        Globals.logInfo.logger.info("get application table")
        kareList = self.getKarePackages()
        print("{P{P{P{" ,kareList.keys())
        androidAppList = self.queryAndroidAppList()
        res = None
        sql = "select id,app_name,b.tagId,cpu_model,no_cpu_model,softStyle,ashStatus\
            from application a INNER join (select appId,group_concat(distinct tagId) as tagId from app_tag where tagStatus=0 and deleted=0 group by appId) b\
            on a.id = b.appId\
            order by download_total desc;"
        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("select application list from application failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        if(res != None and res != []):
            for item in res:
                if(len(item) == 7):
                    appId = str(item[0])
                    pkgname = str(item[1])
                    tagId = str(item[2])
                    cpuModel = item[3]
                    noCpuModel = item[4]
                    softStyle = item[5]
                    ashStatus = item[6]
                    ashVersion = ""
                    if("100" in tagId.split(",")):
                        continue
                    if (softStyle == 6):  # 如果为开明包
                        # 判断是否在开明仓库中
                        continue
                    package = self.get_package_by_name(pkgname)
                    #标签中包含移动应应
                    if(Globals.androidTagId in tagId.split(",")):
                        if(androidAppList != [] and appId not in androidAppList):
                            sql = "delete from application where id=" + str(appId)
                            try:
                                lock.acquire(True)
                                self.cursor.execute(sql)
                            except Exception as e:
                                Globals.CONNECT_DB = False
                                Globals.logError.logger.error("delete application info from application table failed:  " + str(e))
                                if (str(e) == "database is locked"):
                                    self.connect.commit()
                                    return -1
                            finally:
                                lock.release()
                        pass
                    #缺开明包的判断处理
                    else:
                        #若获取到的package为空则表示源里面没有
                        if package is None:
                            #从数据库删除
                            sql = "delete from application where id="+str(appId)
                            try:
                                lock.acquire(True)
                                self.cursor.execute(sql)
                            except Exception as e:
                                Globals.CONNECT_DB = False
                                Globals.logError.logger.error("delete application info from application table failed:  " + str(e))
                                if (str(e) == "database is locked"):
                                    self.connect.commit()
                                    return -1
                            finally:
                                lock.release()
                            continue
                        else:
                            # 检查定向上架字段
                            if(self.checkCpuModel(cpuModel,noCpuModel) == False):
                                # 从数据库删除
                                sql = "delete from application where id=" + str(appId)
                                try:
                                    lock.acquire(True)
                                    self.cursor.execute(sql)
                                except Exception as e:
                                    Globals.CONNECT_DB = False
                                    Globals.logError.logger.error("delete application info from application table failed:  " + str(e))
                                    if (str(e) == "database is locked"):
                                        self.connect.commit()
                                        return -1
                                finally:
                                    lock.release()
                                continue
                            else:
                                #插入前判断应用是否为灰度应用，若为灰度应用ashStatus状态为1
                                if (ashStatus == 1):
                                    # 如果应用是灰度包，则检查包列表中是否存在包的可展示版本
                                    allGrayrelease = True     #默认全部为灰度包
                                    getVersion = False     #是否获取完版本，获取完了拿灰度包版本
                                    for i in range(len(package.versions)):
                                        record = package.versions.__getitem__(i).record
                                        if(getVersion == False):
                                            if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):     #判断关键字是否为true
                                                # 该包为灰度包,判断是否存在灰度策略
                                                if(self.judgAshPackage(appId,package.versions.__getitem__(i).version)):      #存在灰度策略
                                                    allGrayrelease = False
                                                    packageStatus = 0
                                                    if (package.is_installed and package.candidate.installed_size != 0):
                                                        size = package.candidate.installed_size
                                                    else:
                                                        size = package.candidate.size
                                                    version = package.versions.__getitem__(i).version
                                                    aptDescription = package.candidate.description
                                                    aptSummary = package.candidate.summary
                                                    # 包存在则检测包的安装状态
                                                    if (package.is_installed):
                                                        if (self.getPackageUpgradable(appId,pkgname)):
                                                            packageStatus = 2
                                                        else:
                                                            packageStatus = 1
                                                    else:
                                                        #如果应用在已安装kare应用列表
                                                        if(pkgname in kareList):
                                                            installedVersion = kareList[pkgname]["version"]
                                                            # 包存在则检测包的安装状态
                                                            if (apt.apt_pkg.version_compare(version, installedVersion) > 0):
                                                                packageStatus = 2
                                                            else:
                                                                packageStatus = 1
                                                    sql = "update application set packageStatus=?,size=?,version=?,aptDescription=?,aptSummary=? where id=? "
                                                    try:
                                                        lock.acquire(True)
                                                        self.cursor.execute(sql, (
                                                            packageStatus, size, version, aptDescription, aptSummary,
                                                            appId))
                                                    except Exception as e:
                                                        Globals.logError.logger.error(
                                                            "update packageStatus msg: %s" % str(e))
                                                    finally:
                                                        lock.release()
                                                    getVersion = True
                                            else:
                                                # 该版本不为灰度包，写入预处理数据
                                                allGrayrelease = False
                                                packageStatus = 0
                                                size = aptDescription = version = aptSummary = ""
                                                if (package.is_installed and package.candidate.installed_size != 0):
                                                    size = package.candidate.installed_size
                                                else:
                                                    size = package.candidate.size
                                                version = package.versions.__getitem__(i).version
                                                aptDescription = package.candidate.description
                                                aptSummary = package.candidate.summary
                                                # 包存在则检测包的安装状态
                                                if (package.is_installed):
                                                    if(self.getPackageUpgradable(appId,pkgname)):
                                                        packageStatus = 2
                                                    else:
                                                        packageStatus = 1
                                                else:
                                                    # 如果应用在已安装kare应用列表
                                                    if (pkgname in kareList):
                                                        installedVersion = kareList[pkgname]["version"]
                                                        # 包存在则检测包的安装状态
                                                        if (apt.apt_pkg.version_compare(version, installedVersion) > 0):
                                                            packageStatus = 2
                                                        else:
                                                            packageStatus = 1

                                                sql = "update application set packageStatus=?,size=?,version=?,aptDescription=?,aptSummary=? where id=? "
                                                try:
                                                    lock.acquire(True)
                                                    self.cursor.execute(sql, (
                                                        packageStatus, size, version, aptDescription, aptSummary,
                                                        appId))
                                                except Exception as e:
                                                    Globals.logError.logger.error(
                                                        "update packageStatus msg: %s" % str(e))
                                                finally:
                                                    lock.release()
                                                getVersion = True
                                        #获取可展示出来的灰度包版本
                                        if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                                            # 该包为灰度包,判断是否存在灰度策略
                                            if (self.judgAshPackage(appId,package.versions.__getitem__(i).version) != True):  # 不存在灰度策略,则表示有灰度包
                                                ashVersion = package.versions.__getitem__(i).version

                                    if(ashVersion != ""):       #存在灰度包版本，将灰度包版本写入数据库
                                        sql = "update application set ashVersion=? where id=? "
                                        try:
                                            lock.acquire(True)
                                            self.cursor.execute(sql, (ashVersion,appId))
                                        except Exception as e:
                                            Globals.logError.logger.error(
                                                "update ashVersion msg: %s" % str(e))
                                        finally:
                                            lock.release()
                                    if (allGrayrelease == True):  # 全是灰度的包
                                        # 从数据库删除
                                        sql = "delete from application where id=" + str(appId)
                                        try:
                                            lock.acquire(True)
                                            self.cursor.execute(sql)
                                        except Exception as e:
                                            Globals.CONNECT_DB = False
                                            Globals.logError.logger.error(
                                                "delete application info from application table failed:  " + str(e))
                                            if (str(e) == "database is locked"):
                                                self.connect.commit()
                                                return -1
                                        finally:
                                            lock.release()
                                        continue
                                else:
                                    packageStatus = 0
                                    size = package.candidate.size
                                    version = package.candidate.version
                                    aptDescription = package.candidate.description
                                    aptSummary = package.candidate.summary
                                    # 如果应用在已安装kare应用列表
                                    if (pkgname in kareList):
                                        installedVersion = kareList[pkgname]["version"]
                                        # 包存在则检测包的安装状态
                                        if (apt.apt_pkg.version_compare(version, installedVersion) > 0):
                                            packageStatus = 2
                                        else:
                                            packageStatus = 1
                                    else:
                                        #包存在则检测包的安装状态
                                        if(package.is_upgradable and package.is_installed):
                                            packageStatus = 2
                                        elif(package.is_installed and package.is_upgradable == False):
                                            packageStatus = 1
                                    sql = "update application set packageStatus=?,size=?,version=?,aptDescription=?,aptSummary=? where id=? "
                                    try:
                                        lock.acquire(True)
                                        self.cursor.execute(sql, (packageStatus,size,version,aptDescription,aptSummary, appId))
                                    except Exception as e:
                                        Globals.logError.logger.error("update packageStatus msg: %s" % str(e))
                                    finally:
                                        lock.release()

                            pass
            self.connect.commit()
        pass

    #判断灰度包是否显示
    def judgAshPackage(self,appId,version):
        res = []
        strategyIdList = []
        sql = "select id from strategy where enable=1 and deleted=0 and typeId=6"
        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("select from strategy failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        for item in res:
            if (len(item) > 0):
                strategyIdList.append(item[0])
        res = []
        sql = "select strategyId from ash_package_version where appId=\"" + appId + "\" and version=\"" + version+"\""
        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("select from ash_package_version failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        if (res != None and res != []):
            #存在对应的灰度策略，且策略生效该灰度包显示
            for item in res:
                if (len(item) == 1):
                    if(item[0] in strategyIdList):
                        return True
        return False

    def getPackageUpgradable(self,appId,packageName):
        try:
            package = self.aptCache[packageName.lower()]
            version = package.candidate.version
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包,判斷机器是否存在灰度策略
                    if (self.judgAshPackage(appId, package.versions.__getitem__(i).version)):  # 存在灰度策略
                        version = package.versions.__getitem__(i).version
                        break
                    else:
                        continue
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    break
            installedVersion = ""
            try:
                # 使用dpkg命令查询包版本
                output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

                # 输出是多行的，我们需要找到包含版本信息的那一行
                for line in output.decode('utf-8').split('\n'):
                    if packageName in line:
                        # 版本信息通常在输出的第三列
                        installedVersion = line.split()[2]

            except subprocess.CalledProcessError as e:
                # 如果命令执行出错，打印错误信息并返回None
                Globals.logError.logger.error("get " + packageName + " installed version list error : %s" % str(e))

            if apt.apt_pkg.version_compare(version, installedVersion) >0 :
                return True
            else:
                return False
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" update status error : %s" % str(e))
        return False

    def queryAndroidAppList(self):
        res = None
        sql = "select appid from kydroid_app_list"
        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("select from kydroid_app_list failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1

        androidAppList = []
        if(res != None and res !=[]):
            for item in res:
                if(len(item) == 1):
                    androidAppList.append(item[0])
        return androidAppList

    # 预处理app_tag表格
    def queryAppTag(self):
        res = None
        sql = "delete from app_tag where appId not in (select id from application) or (deleted != 0 or tagStatus != 0)"
        try:
            lock.acquire(True)
            self.cursor.execute(sql)
            self.connect.commit()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("delete from app_tag failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        finally:
            lock.release()

    #预处理广告模块
    def queryAdList(self):
        res = None
        sql = "select id,softwareId from ad_list"
        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("select from ad_list failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1

        if (res != None and res != []):
            for item in res:
                if (len(item) >= 2):
                    adId = item[0]
                    softwareId = str(item[1])
                    #带逗号的为专题数据
                    if(len(softwareId.split(",")) == 1 or softwareId == "" or softwareId == None):
                        sql = "delete from ad_list where id="+str(adId) +" and (softwareId not in (select id from application))"
                        try:
                            lock.acquire(True)
                            self.cursor.execute(sql)
                            self.connect.commit()
                        except Exception as e:
                            Globals.CONNECT_DB = False
                            Globals.logError.logger.error("delete from ad_list failed:  " + str(e))
                            if (str(e) == "database is locked"):
                                return -1
                        finally:
                            lock.release()
        pass

    def queryAdLocation(self):
        #删除不会显示的广告位
        sql = "delete from ad_locations where id not in (select locationId from ad_list) and template != 0"
        try:
            lock.acquire(True)
            self.cursor.execute(sql)
            self.connect.commit()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("delete from ad_locations failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        finally:
            lock.release()

        #删除不存在的广告位的国际化数据
        sql = "delete from ad_locations_internationalization where locationId not in (select id from ad_locations)"
        try:
            lock.acquire(True)
            self.cursor.execute(sql)
            self.connect.commit()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("delete from ad_locations_internationalization failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        finally:
            lock.release()

    #预处理搜索推荐应用列表
    def querySearchHotApp(self):
        sql = "delete from search_hot_app where softwareId not in (select id from application)"
        try:
            lock.acquire(True)
            self.cursor.execute(sql)
            self.connect.commit()
        except Exception as e:
            Globals.CONNECT_DB = False
            Globals.logError.logger.error("search hot app list failed:  " + str(e))
            if (str(e) == "database is locked"):
                return -1
        finally:
            lock.release()


