#!/usr/bin/python3
# -*- coding: utf-8 -*-

#/*
# * Copyright (C) 2020, KylinSoft Co., Ltd.
# *
# * This program is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <https://www.gnu.org/licenses/>.
# */


#处理包信息的dbus接口

from model.globals import Globals
import dbus
import dbus.service
import apt
import json
import subprocess

INTERFACE = "com.kylin.softwarecenter.packageInfo"
UKPATH = "/com/kylin/softwarecenter/packageInfo"
class PackageInfomation(dbus.service.Object):
    aptCache = None
    def __init__(self,bus, mainloop):
        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)
        dbus.service.Object.__init__(self, self.bus_name, UKPATH)
        self.mainloop = mainloop
        self.initAptCache()
        self.reloadAptCache()
        pass

    def initAptCache(self):
        try:
            self.aptCache = apt.Cache()
            self.aptCache.open()
        except Exception as e:
            Globals.logError.logger.error("init apt cache error : %s"  % str(e))
            self.aptCache = None


    def getPackageUpgradable(self,packageName,ashVersion):
        try:
            package = self.aptCache[packageName.lower()]
            version = package.candidate.version
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包，且包灰度版本与服务端获取的版本一致时
                    if(ashVersion != "" and ashVersion == package.versions.__getitem__(i).version):
                        pass
                    else:
                        version = package.versions.__getitem__(i).version
                        break
                    continue
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    break
            installedVersion = ""
            installedVersion = self.isKarePakcage(packageName)
            try:
                # 使用dpkg命令查询包版本
                output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

                # 输出是多行的，我们需要找到包含版本信息的那一行
                for line in output.decode('utf-8').split('\n'):
                    if packageName in line:
                        # 版本信息通常在输出的第三列
                        installedVersion = line.split()[2]

            except subprocess.CalledProcessError as e:
                # 如果命令执行出错，打印错误信息并返回None
                Globals.logError.logger.error("get " + packageName + " installed version list error : %s" % str(e))

            if installedVersion != "" and apt.apt_pkg.version_compare(version, installedVersion) > 0:
                return True
            else:
                return False
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" update status error : %s" % str(e))
        return False


    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    @dbus.service.method(INTERFACE, in_signature='', out_signature='b')
    def reloadAptCache(self):
        if(self.aptCache != None):
            self.aptCache.close()
            self.aptCache.open()
        return True

    @dbus.service.method(INTERFACE, in_signature='ss', out_signature='s')
    def getPackageInfo(self,packageName,ashVersion):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            if(package.is_installed and package.candidate.installed_size != 0):
                msg["size"] = package.candidate.installed_size
            else:
                msg["size"] = package.candidate.size
            version = package.candidate.version
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包，判断是否为可显示的灰度包
                    if(ashVersion != "" and package.versions.__getitem__(i).version == ashVersion):
                        pass
                    else:
                        version = package.versions.__getitem__(i).version
                        break

                    continue
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    break
            msg["version"] = version
            msg["aptDescription"] = package.candidate.description
            msg["aptSummary"] = package.candidate.summary
            installedVersion = self.isKarePakcage(packageName)
            if(installedVersion == ""):
                msg["installStatue"] = package.is_installed
                msg["isKarePackage"] = False
            else:
                msg["installStatue"] = True
                msg["isKarePackage"] = False
            if(package.is_installed):
                msg["updatable"] = self.getPackageUpgradable(packageName,ashVersion)
            else:
                if(installedVersion): #获取到了应用使用kare安装版本
                    if (apt.apt_pkg.version_compare(package.candidate.version, installedVersion) > 0):
                        msg["updatable"] = True
                    else:
                        msg["updatable"] = False
                    pass
                else:
                    msg["updatable"] = False
        except Exception as e:
            Globals.logError.logger.error("get "+packageName + " packageInfo error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='ss', out_signature='s')
    def getPackageUpdateStatus(self,packageName,ashVersion):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            version = package.candidate.version
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包
                    if (ashVersion != "" and package.versions.__getitem__(i).version == ashVersion):
                        pass
                    else:
                        version = package.versions.__getitem__(i).version
                        break
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    break
            msg["version"] = version
            installedVersion = ""
            installedVersion = self.isKarePakcage(packageName)
            try:
                # 使用dpkg命令查询包版本
                output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

                # 输出是多行的，我们需要找到包含版本信息的那一行
                for line in output.decode('utf-8').split('\n'):
                    if packageName in line:
                        # 版本信息通常在输出的第三列
                        installedVersion = line.split()[2]

            except subprocess.CalledProcessError as e:
                # 如果命令执行出错，打印错误信息并返回None
                Globals.logError.logger.error("get " + packageName + " installed version list error : %s" % str(e))
            msg["installedVersion"] = installedVersion

            if installedVersion != "" and apt.apt_pkg.version_compare(version, installedVersion) > 0:
                msg["updatable"] = True
            else:
                msg["updatable"] = False
            msg["installStatue"] = package.is_installed
            # msg["updatable"] = package.is_upgradable
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" update status error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageInstallStatus(self, packageName):
        msg = {}
        try:
            version = self.isKarePakcage(packageName)
            if(version == ""):
                package = self.aptCache[packageName.lower()]
                if(package):
                    msg["installStatue"] = package.is_installed
                else:
                    msg["installStatue"] = False
            else:
                msg["installStatue"] = True
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" install status error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson


    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageVersion(self, packageName):
        msg = {}
        try:
            version = ""
            package = self.aptCache[packageName.lower()]
            version = package.candidate.version
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (
                        record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包
                    continue
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    break
            msg["version"] = version
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" version error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson
    #
    # @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    # def getPackageVersionList(self, packageName):
    #     msg = {}
    #     version = ""
    #     try:
    #         package = self.aptCache[packageName.lower()]
    #         try:
    #             # 使用dpkg命令查询包版本
    #             output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)
    #
    #             # 输出是多行的，我们需要找到包含版本信息的那一行
    #             for line in output.decode('utf-8').split('\n'):
    #                 if packageName in line:
    #                     # 版本信息通常在输出的第三列
    #                     version = line.split()[2]
    #
    #         except subprocess.CalledProcessError as e:
    #             # 如果命令执行出错，打印错误信息并返回None
    #             Globals.logError.logger.error("get "+packageName+" version list error : %s" % str(e))
    #         msg["installedVersion"] = version
    #         msg["versionList"] = package.versions()
    #     except Exception as e:
    #         Globals.logError.logger.error("get "+packageName+" version list error : %s" % str(e))
    #     msgJson = json.dumps(msg)
    #     return msgJson

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageInstalledVersion(self, packageName):
        msg = {}
        version = ""
        try:
            # 使用dpkg命令查询包版本
            output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

            # 输出是多行的，我们需要找到包含版本信息的那一行
            for line in output.decode('utf-8').split('\n'):
                if packageName in line:
                    # 版本信息通常在输出的第三列
                    version = line.split()[2]
            if(version == ""):
                version = self.isKarePakcage(packageName)
        except subprocess.CalledProcessError as e:
            # 如果命令执行出错，打印错误信息并返回None
            Globals.logError.logger.error("get "+packageName+" installed version list error : %s" % str(e))
        if(version == ""):
            version = self.isKarePakcage(packageName)
        msg["installedVersion"] = version
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='ss', out_signature='s')
    def getPackageVersionList(self, packageName,ashVersion):
        msg = {}
        installedVersion = ""
        try:
            package = self.aptCache[packageName.lower()]
            try:
            # 使用dpkg命令查询包版本
                output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

                # 输出是多行的，我们需要找到包含版本信息的那一行
                for line in output.decode('utf-8').split('\n'):
                    if packageName in line:
                        # 版本信息通常在输出的第三列
                        installedVersion = line.split()[2]

            except subprocess.CalledProcessError as e:
                # 如果命令执行出错，打印错误信息并返回None
                Globals.logError.logger.error("get "+packageName+" installedVersion list error : %s" % str(e))
            if (installedVersion == ""):
                installedVersion = self.isKarePakcage(packageName)
            msg["installedVersion"] = installedVersion
            versionList = []
            for i in range(len(package.versions)):
                record = package.versions.__getitem__(i).record
                if (record.get("Grayrelease") != None and (
                        record.get("Grayrelease") == "true" or record.get("Grayrelease") == "True")):  # 判断关键字是否为true
                    # 该包为灰度包,且为没有下发策略的灰度包
                    if(package.versions.__getitem__(i).version == ashVersion):
                        pass
                    else:
                        version = package.versions.__getitem__(i).version
                        versionList.append(version)
                    continue
                else:
                    # 该版本不为灰度包，写入预处理数据
                    version = package.versions.__getitem__(i).version
                    versionList.append(version)
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" update status error : %" % str(e))
        msg["versionList"] = versionList
        msgJson = json.dumps(msg)
        return msgJson

    def isKarePakcage(self,packageName):
        version = ""
        appList = {}
        package = self.getPackgeByName("kare")
        if (package == None or package.is_installed == False):
            return version
        try:
            output = subprocess.check_output(['kare', '-l'], stderr=subprocess.STDOUT)
            # 输出是多行的，我们需要找到包含版本信息的那一行
            for line in output.decode('utf-8').split('\n'):
                if packageName in line:
                    # 版本信息通常在输出的第三列
                    version = line.split()[1]
                    break
        except subprocess.CalledProcessError as e:
            # 如果命令执行出错，打印错误信息并返回None
            Globals.logError.logger.error("get " + packageName + " installed version list error : %s" % str(e))
        return version

    def getPackgeByName(self,packageName):
        if not packageName:
            return None
        if self.aptCache is None:
            return None
        package = None
        try:
            package = self.aptCache[packageName.lower()]
        except:
            package = None

        if(package is not None):
            try:
                if(package.candidate is None):
                    return None
            except:
                package = None
        return package
